
def _alpha_to_lonlat(alpha):
    """Converts an alpha character to longitude and latitude coefficients.

    Args:
        alpha (str): Alpha character.

    Returns:
        tuple: Longitude and Latitude coefficient in degrees.
    """

    if type(alpha) is not str:
        raise ValueError('Invalid alpha type')

    return (ord(alpha) - 65) * 20 - 180


def _alpha_to_sub(alpha):
    """Converts an alpha character to sub square coefficients.

    Args:
        alpha (str): Alpha character.

    Returns:
        float: Sub square coefficient in degrees.
    """

    if type(alpha) is not str:
        raise ValueError('Invalid alpha type')

    return (ord(alpha) - 65 + 0.5) / 12


def gl_to_latlon(gridlocator):
    """Converts a grid locator to latitude and longitude.

    Args:
        gridlocator (str): Grid locator.

    Returns:
        tuple: Latitude and Longitude in degrees.
    """
    if len(gridlocator) < 4 or len(gridlocator) % 2 != 0:
        raise ValueError('Invalid GL length')

    gridlocator = gridlocator.upper()

    # South west corner of the FIELD - first two characters
    lon = _alpha_to_lonlat(gridlocator[0])
    lat = _alpha_to_lonlat(gridlocator[1]) / 2

    if len(gridlocator) < 4:
        lon += 10
        lat += 5
    elif len(gridlocator) < 6:
        # Square - next two characters
        lon += int(gridlocator[2]) * 2 + 1
        lat += int(gridlocator[3]) * 1 + 0.5
    elif len(gridlocator) == 6:
        # subsquare - next two characters
        lon += int(gridlocator[2]) * 2 + _alpha_to_sub(gridlocator[4])
        lat += int(gridlocator[3]) * 1 + _alpha_to_sub(gridlocator[5]) / 2

    return (lat, lon)


def latlon_to_gl(latitude, longitude, fourdigit=False):
    """Converts latitude and longitude to a grid locator.

    Args:
        Latitude and Longitude in degrees.
        `fourdigit`: if True, returns 4-digit grid square

    Returns:
        gridlocator (str): Grid locator.
    """

    if latitude < -90 or latitude > 90:
        raise ValueError('latitude over range of value.')
    if longitude < -180 or longitude > 180:
        raise ValueError('longitude over range of value.')

    gl = []
    str_gl = ''

    # first digit
    idx = int((180 + (longitude // 20 * 20)) / 20)
    tmp_lon1 = -180 + idx * 20
    gl.append(chr(idx + ord('A')))

    # second digit
    idx = int((90 + (latitude // 10 * 10)) / 10)
    tmp_lat1 = -90 + idx * 10
    gl.append(chr(idx + ord('A')))

    # third digit
    idx = int((longitude - tmp_lon1) // 2)
    tmp_lon2 = (longitude - tmp_lon1) // 2 * 2 + tmp_lon1
    gl.append(str(idx))

    # fourth digit
    idx = int(latitude - (latitude // 10 * 10) // 1)
    tmp_lat2 = (latitude - tmp_lat1) // 1 * 1 + tmp_lat1
    gl.append(str(idx))

    if not fourdigit:
        # fifth digit
        idx = int((longitude - tmp_lon2) // (2/24))
        gl.append(chr(idx + ord('a')))

        # sixth digit
        idx = int((latitude - tmp_lat2) // (1/24))
        gl.append(chr(idx + ord('a')))

    for s in gl:
        str_gl += s

    return str_gl


def main():
    pass


if __name__ == '__main__':
    main()



import pytest

from adiftools import adiftools


@pytest.fixture
def prep_instance():
    at = adiftools.ADIFParser()
    file_path = 'tests/sample.adi'
    _ = at.read_adi(file_path)
    return at


@pytest.fixture
def prep_data():
    at = adiftools.ADIFParser()
    file_path = 'tests/sample.adi'
    df = at.read_adi(file_path)
    return df


def test_read_adi(prep_data):
    ''' test adif DataFrame '''
    assert prep_data.shape == (126, 14)
    assert prep_data.columns.tolist() == [
        'CALL', 'MODE', 'RST_SENT', 'RST_RCVD',
        'QSO_DATE', 'TIME_ON', 'QSO_DATE_OFF',
        'TIME_OFF', 'BAND', 'FREQ', 'STATION_CALLSIGN',
        'MY_GRIDSQUARE', 'COMMENT', 'GRIDSQUARE']


def test_plot_monthly(prep_instance):
    prep_instance.plot_monthly('tests/monthly_qso_test.png')
    assert True


def test_plot_band_percentage(prep_instance):
    prep_instance.plot_band_percentage('tests/percentage_band_test.png')
    assert True


def test_number_of_records(prep_instance):
    assert prep_instance.number_of_records == 126


@pytest.mark.parametrize(
    # variables
    [
        'data_in',
        'expected_data',
    ],
    # values
    [
        # test cases
        pytest.param('PM85kg', (35.2781423, 136.8735481)),
        pytest.param('PM95pl', (35.4913535, 139.2841430)),
        pytest.param('PM95vq', (35.6812362, 139.7671248)),
        pytest.param('PM53fo', (33.5849988, 130.4490906)),
        pytest.param('PL36te', (26.2001297, 127.6466452)),
        pytest.param('QN00ir', (40.7354587, 140.6904126)),
        pytest.param('QN02us', (42.7791317, 141.6866364)),
        pytest.param('QN01js', (41.7757043, 140.8158222)),
        pytest.param('PM74rs', (34.7861612, 135.4380483)),
        pytest.param('PM63it', (33.8276948, 132.7003773)),
    ]
)
def test_gl2latlon(data_in, expected_data):
    '''Test gridlocator to latitude and longitude conversion.'''

    ERROR_THRESHOLD_COEFFICIENT = 0.55

    coordinates = adiftools.gl_to_latlon(data_in)
    lat_min = expected_data[0] - 1/24 * ERROR_THRESHOLD_COEFFICIENT
    lat_max = expected_data[0] + 1/24 * ERROR_THRESHOLD_COEFFICIENT
    lon_min = expected_data[1] - 1/12 * ERROR_THRESHOLD_COEFFICIENT
    lon_max = expected_data[1] + 1/12 * ERROR_THRESHOLD_COEFFICIENT

    assert (lat_min <= coordinates[0] <= lat_max and
            lon_min <= coordinates[1] <= lon_max)


@pytest.mark.parametrize(
    # variables
    [
        'expected_data',
        'data_in',
    ],
    # values
    [
        # test cases
        pytest.param('PM85kg', (35.2781423, 136.8735481)),
        pytest.param('PM95pl', (35.4913535, 139.2841430)),
        pytest.param('PM95vq', (35.6812362, 139.7671248)),
        pytest.param('PM53fo', (33.5849988, 130.4490906)),
        pytest.param('PL36te', (26.2001297, 127.6466452)),
        pytest.param('QN00ir', (40.7354587, 140.6904126)),
        pytest.param('QN02us', (42.7791317, 141.6866364)),
        pytest.param('QN01js', (41.7757043, 140.8158222)),
        pytest.param('PM74rs', (34.7861612, 135.4380483)),
        pytest.param('PM63it', (33.8276948, 132.7003773)),
        pytest.param('QN01', (41.7757043, 140.8158222, True)),
        pytest.param('PM74', (34.7861612, 135.4380483, True)),
        pytest.param('PM63', (33.8276948, 132.7003773, True)),
        pytest.param('QN01js', (41.7757043, 140.8158222, False)),
        pytest.param('PM74rs', (34.7861612, 135.4380483, False)),
        pytest.param('PM63it', (33.8276948, 132.7003773, False)),
    ]
)
def test_latlon2gl(data_in, expected_data):
    ''' test latitude and longitude to grid locator '''

    if len(data_in) == 3:
        gridlocator = adiftools.latlon2gl(data_in[0], data_in[1], data_in[2])
    else:
        gridlocator = adiftools.latlon2gl(data_in[0], data_in[1])

    assert gridlocator == expected_data


# TODO: use test fixture to create a temporary file



import re


def is_ja_call(call_sign: str):
    # input data check
    if type(call_sign) is not str:
        raise TypeError('Call sign must be string')

    if len(call_sign) < 4:
        raise ValueError('Call sign must be at least 4 characters long')

    pattern = r"^(J[A-S]|[78][J-N])"
    match = re.match(pattern, call_sign)
    return bool(match)



import pytest

import adiftools.callsign as cs


@pytest.mark.parametrize(
    "callsign, expected",
    [
        ("JA1ABC", True),
        ("7K1XYZ", True),
        ("8L1DEF", True),
        ("JTA1ABC", False),
        ("7Z1XYZ", False),
        ("9J1DEF", False),
    ],
)
def test_is_ja_call(callsign, expected):
    assert cs.is_ja_call(callsign) == expected


@pytest.mark.parametrize(
    "callsign, expected",
    [
        ("", pytest.raises(ValueError)),
        ("JA", pytest.raises(ValueError)),
        ("7", pytest.raises(ValueError)),
        (7, pytest.raises(TypeError)),

    ],
)
def test_error_is_ja_call(callsign, expected):
    with expected as e:
        assert cs.is_ja_call(callsign) == e



from setuptools import setup, find_packages

setup(
    name='adiftools',
    version='0.1.2',
    description='ADIF file utilities',
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='JS2IIU',
    author_email='info@js2iiu.com',
    maintainer='JS2IIU',
    maintainer_email='info@js2iiu.com',
    url='https://github.com/JS2IIU-MH/adiftools-dev',
    download_url='https://github.com/JS2IIU-MH/adiftools-dev',
    packages=find_packages(
        include=['adiftools', 'adiftools.*'],
        exclude=['test/']),
    # package_dir = {"": "adiftools"},
    install_requires=open('requirements.txt').readlines(),
    keywords=['adif'],
    license='MIT',
)



from adiftools import adiftools
from adiftools import adifgraph


df = adiftools.ADIFParser().read_adi('tests/sample.adi')


def test_monthly_qso():
    adifgraph.monthly_qso(df, 'tests/monthly_qso.png')
    assert True


def test_band_percentage():
    adifgraph.band_percentage(df, 'tests/percentage_band.png')
    assert True



import pytest
from adiftools import gridlocator as gl


@pytest.mark.parametrize(
    # variables
    [
        'data_in',
        'expected_data',
    ],
    # values
    [
        # test cases
        pytest.param('PM85kg', (35.2781423, 136.8735481)),
        pytest.param('PM95pl', (35.4913535, 139.2841430)),
        pytest.param('PM95vq', (35.6812362, 139.7671248)),
        pytest.param('PM53fo', (33.5849988, 130.4490906)),
        pytest.param('PL36te', (26.2001297, 127.6466452)),
        pytest.param('QN00ir', (40.7354587, 140.6904126)),
        pytest.param('QN02us', (42.7791317, 141.6866364)),
        pytest.param('QN01js', (41.7757043, 140.8158222)),
        pytest.param('PM74rs', (34.7861612, 135.4380483)),
        pytest.param('PM63it', (33.8276948, 132.7003773)),
    ]
)
def test_gl_to_latlon(data_in, expected_data):
    '''Test gridlocator to latitude and longitude conversion.'''

    ERROR_THRESHOLD_COEFFICIENT = 0.55

    coordinates = gl.gl_to_latlon(data_in)
    lat_min = expected_data[0] - 1/24 * ERROR_THRESHOLD_COEFFICIENT
    lat_max = expected_data[0] + 1/24 * ERROR_THRESHOLD_COEFFICIENT
    lon_min = expected_data[1] - 1/12 * ERROR_THRESHOLD_COEFFICIENT
    lon_max = expected_data[1] + 1/12 * ERROR_THRESHOLD_COEFFICIENT

    assert (lat_min <= coordinates[0] <= lat_max and
            lon_min <= coordinates[1] <= lon_max)


@pytest.mark.parametrize(
    # variables
    [
        'expected_data',
        'data_in',
    ],
    # values
    [
        # test cases
        pytest.param('PM85kg', (35.2781423, 136.8735481)),
        pytest.param('PM95pl', (35.4913535, 139.2841430)),
        pytest.param('PM95vq', (35.6812362, 139.7671248)),
        pytest.param('PM53fo', (33.5849988, 130.4490906)),
        pytest.param('PL36te', (26.2001297, 127.6466452)),
        pytest.param('QN00ir', (40.7354587, 140.6904126)),
        pytest.param('QN02us', (42.7791317, 141.6866364)),
        pytest.param('QN01js', (41.7757043, 140.8158222)),
        pytest.param('PM74rs', (34.7861612, 135.4380483)),
        pytest.param('PM63it', (33.8276948, 132.7003773)),
        pytest.param('QN01', (41.7757043, 140.8158222, True)),
        pytest.param('PM74', (34.7861612, 135.4380483, True)),
        pytest.param('PM63', (33.8276948, 132.7003773, True)),
        pytest.param('QN01js', (41.7757043, 140.8158222, False)),
        pytest.param('PM74rs', (34.7861612, 135.4380483, False)),
        pytest.param('PM63it', (33.8276948, 132.7003773, False)),
    ]
)
def test_latlon_to_gl(data_in, expected_data):
    ''' test latitude and longitude to grid locator '''

    if len(data_in) == 3:
        gridlocator = gl.latlon_to_gl(data_in[0], data_in[1], data_in[2])
    else:
        gridlocator = gl.latlon_to_gl(data_in[0], data_in[1])

    assert gridlocator == expected_data
